unit logcont;

{$mode objfpc}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics,
  Dialogs, StrUtils, dateutils,
  lazutf8sysutils, SynMemo, Sensors, //Defines NOWUTC.
  fileview, //For opening log files.
  dlheader, //For Timezone conversions.
  appsettings, //For saving and restoring hotkeys.
  LCLType, // For kirtual key definitions used for hotkeys.
  Math, //for float identifier
  StdCtrls, ExtCtrls, Buttons, Grids, header_utils;

type

  { TFormLogCont }

  TFormLogCont = class(TForm)
    AnalogSensor1: TAnalogSensor;
    OpenFileButton: TBitBtn;
    AnnotateButton: TButton;
    AnnotateEdit: TEdit;
    AnnotationGroupBox: TGroupBox;
    PersistentCheckBox: TCheckBox;
    PendingHotKey: TEdit;
    PendingLabel: TLabel;
    SynchronizedCheckBox: TCheckBox;
    EditHotkeysCheckBox: TCheckBox;
    RecordsMissedLabel: TLabel;
    RecordsViewLabel: TLabel;
    LogFileNameText: TEdit;
    LogfileNameLabel: TLabel;
    Labelinfile: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Labelfile: TLabel;
    LabelIn: TLabel;
    Label2: TLabel;
    LCTrigSeconds: TLabeledEdit;
    LCTrigMinutes: TLabeledEdit;
    NextRecordAt: TStaticText;
    CurrentTime: TStaticText;
    NextRecordIn: TStaticText;
    RecordsLogged: TStaticText;
    FilesLogged: TStaticText;
    CloseButton: TButton;
    OpenLogDialog: TOpenDialog;
    RecordsViewMemo: TSynMemo;
    StartButton: TBitBtn;
    LCTriggerGroup: TRadioGroup;
    RecordsMissed: TStaticText;
    StopButton: TBitBtn;
    FineTimer: TTimer;
    RecordsViewSynMemo: TSynMemo;
    HotkeyStringGrid: TStringGrid;
    procedure AnnotateButtonClick(Sender: TObject);
    //procedure AnnotateEditKeyPress(Sender: TObject; var Key: char);
    procedure OpenFileButtonClick(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
    procedure EditHotkeysCheckBoxChange(Sender: TObject);
    procedure FineTimerTimer(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: word; Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure PersistentCheckBoxChange(Sender: TObject);
    procedure StartButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure HotkeyStringGridKeyUp(Sender: TObject; var Key: word; Shift: TShiftState);
    procedure HotkeyStringGridSelectEditor(Sender: TObject; aCol, aRow: integer;
      var Editor: TWinControl);
    procedure SynchronizedCheckBoxChange(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    procedure CustomizeEdit(ACol, ARow: integer; Editor: TStringCellEditor);
    procedure CheckSynchronized();
  end;

var
  FormLogCont: TFormLogCont;
  LogTimePreset, LogTimeCurrent: integer;
  LCLoggedCount: integer; // Number of logged records taken during this session
  LCLogFileCount: integer; //Number of logfiles created during this session
  DesiredLogTime: TDateTime;
  StopRecording: boolean;
  Setting: string; //Setting of recorder
  Recording: boolean; //Flag to indicate that record is being logged
  RecordingMode: boolean = False; //Indicate that user has pressed Record button
  TimerBusy: boolean = False; //Flag to indicate that Timer event is being handled

  //Used to check 24 hr log file rollover
  LCStartFileTime: TDateTime;

  RecordsMissedCount: integer;
  //Determined while recording records, incrememts if meter does not respond.

  OldSecond: word; //Previous second value used to check timer rollover.

  AnnotateText: string;
  HotKeyCodes: array[1..100] of word;

implementation

uses Unit1;

function keyslist(Key: word): string;
type
  TSendKey = record
    Name: ShortString;
    VKey: byte;
  end;
const
  {Array of keys that SendKeys recognizes.

  If you add to this list, you must be sure to keep it sorted alphabetically
  by Name because a binary search routine is used to scan it.

  http://delphiworld.narod.ru/base/sendkeys_vb.html}

  MaxSendKeyRecs = 77;
  SendKeyRecs: array[1..MaxSendKeyRecs] of TSendKey =
    (
    (Name: 'F1'; VKey: VK_F1),
    (Name: 'F2'; VKey: VK_F2),
    (Name: 'F3'; VKey: VK_F3),
    (Name: 'F4'; VKey: VK_F4),
    (Name: 'F5'; VKey: VK_F5),
    (Name: 'F6'; VKey: VK_F6),
    (Name: 'F7'; VKey: VK_F7),
    (Name: 'F8'; VKey: VK_F8),
    (Name: 'F9'; VKey: VK_F9),
    (Name: 'F10'; VKey: VK_F10),
    (Name: 'F11'; VKey: VK_F11),
    (Name: 'F12'; VKey: VK_F12),
    (Name: 'F13'; VKey: VK_F13),
    (Name: 'F14'; VKey: VK_F14),
    (Name: 'F15'; VKey: VK_F15),
    (Name: 'F16'; VKey: VK_F16),
    (Name: 'F17'; VKey: VK_F17),
    (Name: 'F18'; VKey: VK_F18),
    (Name: 'F19'; VKey: VK_F19),
    (Name: 'F20'; VKey: VK_F20),
    (Name: 'F21'; VKey: VK_F21),
    (Name: 'F22'; VKey: VK_F22),
    (Name: 'F23'; VKey: VK_F23),
    (Name: 'F24'; VKey: VK_F24),
    (Name: '0'; VKey: VK_0),
    (Name: '1'; VKey: VK_1),
    (Name: '2'; VKey: VK_2),
    (Name: '3'; VKey: VK_3),
    (Name: '4'; VKey: VK_4),
    (Name: '5'; VKey: VK_5),
    (Name: '6'; VKey: VK_6),
    (Name: '7'; VKey: VK_7),
    (Name: '8'; VKey: VK_8),
    (Name: '9'; VKey: VK_9),
    (Name: 'A'; VKey: VK_A),
    (Name: 'B'; VKey: VK_B),
    (Name: 'C'; VKey: VK_C),
    (Name: 'D'; VKey: VK_D),
    (Name: 'E'; VKey: VK_E),
    (Name: 'F'; VKey: VK_F),
    (Name: 'G'; VKey: VK_G),
    (Name: 'H'; VKey: VK_H),
    (Name: 'I'; VKey: VK_I),
    (Name: 'J'; VKey: VK_J),
    (Name: 'K'; VKey: VK_K),
    (Name: 'L'; VKey: VK_L),
    (Name: 'M'; VKey: VK_M),
    (Name: 'N'; VKey: VK_N),
    (Name: 'O'; VKey: VK_O),
    (Name: 'P'; VKey: VK_P),
    (Name: 'Q'; VKey: VK_Q),
    (Name: 'R'; VKey: VK_R),
    (Name: 'S'; VKey: VK_S),
    (Name: 'T'; VKey: VK_T),
    (Name: 'U'; VKey: VK_U),
    (Name: 'V'; VKey: VK_V),
    (Name: 'W'; VKey: VK_W),
    (Name: 'X'; VKey: VK_X),
    (Name: 'Y'; VKey: VK_Y),
    (Name: 'Z'; VKey: VK_Z),
    (Name: 'SPACE'; VKey: VK_SPACE),
    (Name: 'NUMPAD0'; VKey: VK_NUMPAD0),
    (Name: 'NUMPAD1'; VKey: VK_NUMPAD1),
    (Name: 'NUMPAD2'; VKey: VK_NUMPAD2),
    (Name: 'NUMPAD3'; VKey: VK_NUMPAD3),
    (Name: 'NUMPAD4'; VKey: VK_NUMPAD4),
    (Name: 'NUMPAD5'; VKey: VK_NUMPAD5),
    (Name: 'NUMPAD6'; VKey: VK_NUMPAD6),
    (Name: 'NUMPAD7'; VKey: VK_NUMPAD7),
    (Name: 'NUMPAD8'; VKey: VK_NUMPAD8),
    (Name: 'NUMPAD9'; VKey: VK_NUMPAD9),
    (Name: 'MULTIPLY'; VKey: VK_MULTIPLY),
    (Name: 'ADD'; VKey: VK_ADD),
    (Name: 'SEPARATOR'; VKey: VK_SEPARATOR),
    (Name: 'SUBTRACT'; VKey: VK_SUBTRACT),
    (Name: 'DECIMAL'; VKey: VK_DECIMAL),
    (Name: 'DIVIDE'; VKey: VK_DIVIDE)
    );

var
  A: TSendKey;

begin
  keyslist := '';
  for A in SendKeyRecs do
  begin
    if Key = A.VKey then
      keyslist := A.Name;
  end;

end;



function Sec2DHMS(InSeconds: integer): string;
var
  Days: integer;
  Hours: integer;
  InHours: integer;
  Minutes: integer;
  InMinutes: integer;
  Seconds: integer;
begin
  Seconds := InSeconds mod 60;
  InMinutes := InSeconds div 60;

  Minutes := InMinutes mod 60;
  InHours := InMinutes div 60;

  Hours := InHours mod 24;
  Days := InHours div 24;

  Sec2DHMS := Format('%.1dd %.2d:%.2d:%.2d', [Days, Hours, Minutes, Seconds]);
end;

procedure OnTheClock(const ThisMoment: TDateTime; const Granularity: integer);
{This utility gets the next time from now based on a granularity setting,
 i.e for determining log time and difference for logging every x minutes}
begin
  // Determine new record time and calculate time until record
  DesiredLogTime := ThisMoment; //Copy current time for manipulation
  DesiredLogTime := IncMinute(DesiredLogTime, Granularity -
    MinuteOf(ThisMoment) mod Granularity);
  //Replace seconds with 0
  DesiredLogTime := RecodeSecond(DesiredLogTime, 0);
  FormLogCont.NextRecordAt.Caption :=
    FormatDateTime('yyyy-mm-dd hh:nn:ss', DesiredLogTime);
  LogTimeCurrent := SecondsBetween(ThisMoment, DesiredLogTime);
end;

procedure LogOneReading();
var
  pieces: TStringList;
  Result: string;
  ThisMoment, ThisMomentUTC: TDateTime;
  subfix: ansistring;

  function CheckRedordCount(): boolean;
  begin
    if ((SelectedModel = 1) and (pieces.Count = 8)) or
      ((SelectedModel <> 1) and (pieces.Count = 6)) then
      CheckRedordCount := True
    else
      CheckRedordCount := False;
  end;


  procedure WriteRecord(Special: string = '');
  var
    ComposeString: string;
    Temperature, Darkness: Float;
  begin
    ThisMomentUTC := NowUTC;
    ThisMoment := dlheader.ptz.GMTToLocalTime(ThisMomentUTC,
      DLHeaderForm.TZLocationBox.Text, subfix);

    Application.ProcessMessages;

    //Create new logfile if over 24hr UTC time.
    if (DayOf(LCStartFileTime) <> DayOf(ThisMoment)) then
    begin
      // Open new file and store header
      WriteDLHeader('LE', Format('Logged continuously %s.', [setting]));

      LCStartFileTime := ThisMoment;

      // Display number of logfiles
      Inc(LCLogFileCount);
      FormLogCont.FilesLogged.Caption := IntToStr(LCLogFileCount);
      if LCLogFileCount = 1 then
        FormLogCont.Labelfile.Caption := 'file'
      else
        FormLogCont.Labelfile.Caption := 'files';
    end
    else
    begin
      AssignFile(DLRecFile, LogFileName);
      Append(DLRecFile); //Open file for appending
      SetTextLineEnding(DLRecFile, #13#10);
    end;

    // Display LogFileNameText path
    FormLogCont.LogFileNameText.Caption := LogFileName;

    if (Special = '') then
    begin
      if SelectedModel = 1 then //ADA device
      begin
        // YYYY-MM-DDTHH:mm:ss.fff;YYYY-MM-DDTHH:mm:ss.fff;Hz;Count1;Time1;Count2;Time2
        //r,0000000238Hz,0000000000c,0000000.000s,0000000000c,0000000.000s,000,109
        ComposeString :=
          FormatDateTime('yyyy-mm-dd"T"hh:nn:ss.zzz";"', NowUTC) //Date UTC
          + FormatDateTime('yyyy-mm-dd"T"hh:nn:ss.zzz";"', Now) //Date Local
          + Format('%d;', [StrToIntDef(AnsiMidStr(pieces.Strings[1], 1, 10), 0)])
          //Frequency
          + Format('%d;', [StrToIntDef(AnsiMidStr(pieces.Strings[2], 1, 10), 0)])
          //Counter1
          + Format('%1.3f;',
          [StrToFloatDef(AnsiMidStr(pieces.Strings[3], 1, 11), 0)]) //Time1
          + Format('%d;', [StrToIntDef(AnsiMidStr(pieces.Strings[4], 1, 10), 0)])
          //Counter2
          + Format('%1.3f;', [StrToFloatDef(AnsiMidStr(pieces.Strings[5], 1, 11), 0)])
        //Time2
        ;
      end
      else //must be LE or LU device
      begin

        { Pull in values }
        Temperature := StrToFloatDef(
          AnsiLeftStr(pieces.Strings[5], Length(pieces.Strings[5]) - 1), 0, FPointSeparator);
        Darkness := StrToFloatDef(AnsiLeftStr(pieces.Strings[1],
          Length(pieces.Strings[1]) - 1), 0, FPointSeparator);

        // YYYY-MM-DDTHH:mm:ss.fff;YYYY-MM-DDTHH:mm:ss.fff;Celsius;number;Hz;mag/arcsec^2')
        ComposeString :=
          FormatDateTime('yyyy-mm-dd"T"hh:nn:ss.zzz";"', ThisMomentUTC) //Date UTC
          //+ FormatDateTime('yyyy-mm-dd"T"hh:nn:ss.zzz";"', Now) //Date Local
          //+ FormatDateTime('yyyy-mm-dd"T"hh:nn:ss.zzz";"', DesiredLogTime)
          + FormatDateTime('yyyy-mm-dd"T"hh:nn:ss.zzz";"', ThisMoment)
          //Date Local (calculated)
          + FormatFloat('##0.0', Temperature, FPointSeparator) + ';' //Temperature
          + Format('%d;', [StrToIntDef(AnsiLeftStr(pieces.Strings[3],
          length(pieces.Strings[3]) - 1), 0)]) //counts
          + Format('%d;', [StrToIntDef(AnsiLeftStr(pieces.Strings[2],
          length(pieces.Strings[2]) - 2), 0)]) //Hz
          + FormatFloat('#0.00', Darkness, FPointSeparator) //mpsas value
        ;

      end;
      if (Length(AnnotateText) > 0) then
      begin
        ComposeString := ComposeString + ';' + AnnotateText;
        if (not FormLogCont.PersistentCheckBox.Checked) then
        begin
          AnnotateText := '';
          FormLogCont.PendingHotKey.Text := '';
        end;
      end;

    end //end of "special" text check when no data was available (missed record).
    else
    begin
      ComposeString :=
        FormatDateTime('yyyy-mm-dd"T"hh:nn:ss.zzz";"', NowUTC) //Date UTC
        + FormatDateTime('yyyy-mm-dd"T"hh:nn:ss.zzz";"', Now) //Date Local
        + Special;
    end;

    Writeln(DLRecFile, ComposeString);
    FormLogCont.RecordsViewSynMemo.Append(ComposeString);
    //Ensure that latest append is displayed at bottom of window.
    FormLogCont.RecordsViewSynMemo.CaretY := FormLogCont.RecordsViewSynMemo.Lines.Count;
    Inc(LCLoggedCount);
    FormLogCont.RecordsLogged.Caption := IntToStr(LCLoggedCount);

  end;

begin
  Recording := True;
  pieces := TStringList.Create;
  pieces.Delimiter := ',';
  pieces.StrictDelimiter := False; //Parse spaces also

  //Try a first time
  Result := GetReading;
  pieces.DelimitedText := Result;
  if CheckRedordCount then
    WriteRecord
  else
  begin //Try a second time
    Result := GetReading;
    pieces.DelimitedText := Result;
    if CheckRedordCount then
      WriteRecord
    else
    begin //Try a second time
      Result := GetReading;
      pieces.DelimitedText := Result;
      if CheckRedordCount then
        WriteRecord
      else
      begin
        Inc(RecordsMissedCount);
        WriteRecord(';;;'); //Empty fields
        FormLogCont.RecordsMissed.Color := clRed;
        FormLogCont.RecordsMissed.Font.Color := clWhite;
        FormLogCont.RecordsMissed.Caption := IntToStr(RecordsMissedCount);
      end;
    end;
  end;

  Flush(DLRecFile);
  Close(DLRecFile);
  Recording := False;
end;

{ TFormLogCont }

procedure TFormLogCont.OpenFileButtonClick(Sender: TObject);
begin
  OpenLogDialog.InitialDir := appsettings.LogsDirectory;
  if OpenLogDialog.Execute then
  begin
    Form2.Memo1.Lines.LoadFromFile(OpenLogDialog.Filename);
    Form2.Show;
  end;

end;

procedure TFormLogCont.AnnotateButtonClick(Sender: TObject);
begin
  //Initiate a recording with annotated text while already in record-mode.
  AnnotateText := FormLogCont.AnnotateEdit.Text;
  PendingHotKey.Text := AnnotateText;
  if (not SynchronizedCheckBox.Checked) then
    LogOneReading;
end;

//procedure TFormLogCont.AnnotateEditKeyPress(Sender: TObject; var Key: char);
//begin
//  // Only annotate if recording.
//  // Annotate when the enter key (#13) is pressed.
//  if ((RecordingMode) and (Key = #13)) then
//    AnnotateButtonClick(Sender);
//end;

procedure TFormLogCont.CloseButtonClick(Sender: TObject);
begin
  Close;
end;

procedure TFormLogCont.EditHotkeysCheckBoxChange(Sender: TObject);
begin
  if EditHotkeysCheckBox.Checked then
  begin
    HotkeyStringGrid.Enabled := True;
    HotkeyStringGrid.Font.Color := clDefault;
  end
  else
  begin
    HotkeyStringGrid.Enabled := False;
    HotkeyStringGrid.Font.Color := clInactiveCaption;
  end;

end;

procedure TFormLogCont.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  if CloseButton.Enabled then
    Close
  else
    StopRecording := True;

end;

procedure TFormLogCont.FormKeyUp(Sender: TObject; var Key: word; Shift: TShiftState);
var
  i: integer;

begin
  if RecordingMode and not (AnnotateEdit.Focused or EditHotkeysCheckBox.Checked) then
  begin
    //Search for key in hotkey list
    //writeln(char(key),' = ',Key);
    keyslist(Key);
    for i := 1 to HotkeyStringGrid.RowCount - 1 do
    begin
      if (Key = HotKeyCodes[i]) then
      begin
        AnnotateText := vConfigurations.ReadString('HotKeys', 'a' +
          IntToStr(i - 1), '');
        PendingHotKey.Text := AnnotateText;
        if (not SynchronizedCheckBox.Checked) then
          LogOneReading
        else
          PendingHotKey.Text := AnnotateText;
      end;
    end;
  end;
end;

procedure TFormLogCont.FormShow(Sender: TObject);
var
  i: integer;
begin
  RecordingMode := False;
  StopButton.Enabled := False;
  StartButton.Enabled := True;
  LogTimePreset := 0;
  LogTimeCurrent := 0;
  NextRecordIn.Caption := '';
  LCLoggedCount := 0;
  RecordsLogged.Caption := IntToStr(LCLoggedCount);
  LCLogFileCount := 0;
  FilesLogged.Caption := IntToStr(LCLogFileCount);
  OldSecond := SecondOf(Now);
  RecordsMissedCount := 0;
  RecordsMissed.Caption := IntToStr(RecordsMissedCount);
  AnnotateButton.Enabled := False;


  for i := 1 to HotkeyStringGrid.RowCount - 1 do
  begin
    HotKeyCodes[i] := StrToIntDef(vConfigurations.ReadString('HotKeys',
      'h' + IntToStr(i - 1), ''), 0);
    HotkeyStringGrid.Cells[0, i] :=
      keyslist(StrToIntDef(vConfigurations.ReadString('HotKeys', 'h' +
      IntToStr(i - 1), ''), 0));
    HotkeyStringGrid.Cells[1, i] :=
      vConfigurations.ReadString('HotKeys', 'a' + IntToStr(i - 1), '');
  end;

  SynchronizedCheckBox.Checked := vConfigurations.ReadBool('HotKeys', 'sync', False);
  PersistentCheckBox.Checked := vConfigurations.ReadBool('HotKeys', 'pers', False);

  CheckSynchronized();

end;

procedure TFormLogCont.PersistentCheckBoxChange(Sender: TObject);
begin
  if (PersistentCheckBox.Checked) then
    vConfigurations.WriteBool('HotKeys', 'pers', True)
  else
    vConfigurations.WriteBool('HotKeys', 'pers', False);
end;

procedure TFormLogCont.FineTimerTimer(Sender: TObject);
var
  ThisMoment: TDateTime;
begin
  // Only trigger out once per second with ~20ms accuracy.
  // This prevents general drift in recordings.

  FineTimer.Enabled := False;

  if (SecondOf(Now) <> OldSecond) then
  begin
    OldSecond := SecondOf(Now);
    if (RecordingMode) then
    begin
      //Gets triggered by the fine timer once per second if RecordingMode is true.

      if (Recording = False) then
        //Prevent more recording while already busy saving a record
      begin
        ThisMoment := RecodeMilliSecond(Now, 0);
        CurrentTime.Caption := FormatDateTime('yyyy-mm-dd hh:nn:ss', ThisMoment);

        // Check if counter has expired while in Seconds or Minutes mode
        case LCTriggerGroup.ItemIndex of
          0..1:
          begin // Seconds or Minutes mode selected
            Dec(LogTimeCurrent);
            if (LogTimeCurrent <= 0) then
            begin
              //LogContTimer.Enabled := False;

              // Log value//* threshold checking
              LogOneReading;

              // Restart counter for continuous logging
              LogTimeCurrent := LogTimePreset;
              NextRecordAt.Caption :=
                FormatDateTime('yyyy-mm-dd hh:nn:ss',
                IncSecond(ThisMoment, LogTimeCurrent));
              NextRecordIn.Caption := Sec2DHMS(LogTimeCurrent);

              RecordsLogged.Caption := IntToStr(LCLoggedCount);
              //LogContTimer.Enabled := True;
            end
            else
              NextRecordIn.Caption := Sec2DHMS(LogTimeCurrent);

          end;

          2..7:
          begin // On-the-clock mode selected
            if (CompareDateTime(ThisMoment, DesiredLogTime) >= 0) then
            begin
              //LogContTimer.Enabled := False;

              // Log value//* threshold checking
              LogOneReading;

              case LCTriggerGroup.ItemIndex of
                2: OnTheClock(ThisMoment, 1); //Every 1 minute
                3: OnTheClock(ThisMoment, 5); //Every 5 minutes
                4: OnTheClock(ThisMoment, 10); //Every 10 minutes
                5: OnTheClock(ThisMoment, 15); //Every 15 minutes
                6: OnTheClock(ThisMoment, 30); //Every 30 minutes
                76: OnTheClock(ThisMoment, 60); //Every hour
              end;

              RecordsLogged.Caption := IntToStr(LCLoggedCount);
              //LogContTimer.Enabled := True;
            end
            else //continue counting
            begin
              LogTimeCurrent := Round(SecondSpan(ThisMoment, DesiredLogTime));
              NextRecordIn.Caption := Sec2DHMS(LogTimeCurrent);
            end;
          end;
        end;

        if StopRecording then
        begin
          RecordingMode := False;
          StopButton.Enabled := False;
          StartButton.Enabled := True;
          LCTrigSeconds.Enabled := True;
          LCTrigMinutes.Enabled := True;
          LogTimePreset := 0;
          LogTimeCurrent := 0;
          NextRecordIn.Caption := '';
          AnnotateButton.Enabled := False;
          EditHotkeysCheckBox.Checked := True;
          EditHotkeysCheckBox.Enabled := False;
          HotkeyStringGrid.Enabled := True;
          HotkeyStringGrid.Font.Color := clDefault;


          CloseButton.Enabled := True;
          LCTriggerGroup.Enabled := True;
        end;
      end; //End of checking Recording flag

    end;

  end;

  //Set length of timer and start it if still recording
  if not StopRecording then
  begin
    FineTimer.Interval := 1000 - MilliSecondOf(Now);
    FineTimer.Enabled := True;
  end;

end;


procedure TFormLogCont.StartButtonClick(Sender: TObject);  //Start recording
var
  ThisMoment: TDateTime;
  LogMinutes: integer;
begin
  StopRecording := False;

  //Grab selected time, convert to seconds, store in LogTimePreset

  //Grab current time:
  ThisMoment := RecodeMilliSecond(Now, 0); //Strip fractional seconds for display
  CurrentTime.Caption := FormatDateTime('yyyy-mm-dd hh:nn:ss', ThisMoment);

  case LCTriggerGroup.ItemIndex of
    0: //Every x seconds (on the second)
    begin
      LogTimePreset := StrToIntDef(LCTrigSeconds.Text, 1);
      LCTrigSeconds.Text := IntToStr(LogTimePreset);
      LogTimeCurrent := LogTimePreset;
      NextRecordAt.Caption := FormatDateTime('yyyy-mm-dd hh:nn:ss',
        IncSecond(ThisMoment, LogTimeCurrent));
      Setting := Format('every %d seconds', [LogTimePreset]);
    end;
    1: //Every x minutes (at 0 seconds)
    begin
      { TODO : ontheclock x minutes }
      LogMinutes := StrToIntDef(LCTrigMinutes.Text, 1);
      LogTimePreset := LogMinutes * 60;
      LCTrigMinutes.Text := IntToStr(LogTimePreset div 60);
      LogTimeCurrent := LogTimePreset;
      NextRecordAt.Caption := FormatDateTime('yyyy-mm-dd hh:nn:ss',
        IncSecond(ThisMoment, LogTimeCurrent));
      if LogMinutes = 1 then
        Setting := Format('every 1 minute', [LogMinutes])
      else
        Setting := Format('every %d minutes', [LogMinutes]);
    end;
    2: //Every 1 minutes
    begin
      OnTheClock(ThisMoment, 1);
      Setting := 'every 1 minute on the minute';
    end;
    3: //Every 5 minutes
    begin
      OnTheClock(ThisMoment, 5);
      Setting := 'every 5 minutes on the 1/12th hour';
    end;
    4://Every 10 minutes
    begin
      OnTheClock(ThisMoment, 10);
      Setting := 'every 10 minutes on the 1/6th hour';
    end;
    5://Every 15 minutes
    begin
      OnTheClock(ThisMoment, 15);
      Setting := 'every 15 minutes on the 1/4 hour';
    end;
    6://Every 30 minutes
    begin
      OnTheClock(ThisMoment, 30);
      Setting := 'every 30 minutes on the 1/2 hour';
    end;
    7://Every hour
    begin
      OnTheClock(ThisMoment, 60);
      Setting := 'every hour on the hour';
    end;
  end;

  NextRecordIn.Caption := Sec2DHMS(LogTimeCurrent);
  RecordingMode := True;
  StopButton.Enabled := True;
  StartButton.Enabled := False;
  CloseButton.Enabled := False;
  LCTrigSeconds.Enabled := False;
  LCTrigMinutes.Enabled := False;
  LCLoggedCount := 0;
  RecordsLogged.Caption := IntToStr(LCLoggedCount);
  LCTriggerGroup.Enabled := False;
  RecordsViewSynMemo.Clear;
  AnnotateButton.Enabled := True;
  EditHotkeysCheckBox.Checked := False;
  EditHotkeysCheckBox.Enabled := True;
  HotkeyStringGrid.Enabled := False;
  HotkeyStringGrid.Font.Color := clInactiveCaption;
  RecordsMissed.Color := clDefault;
  RecordsMissed.Font.Color := clDefault;
  RecordsMissedCount := 0;
  RecordsMissed.Caption := IntToStr(RecordsMissedCount);

  try
    // Open file and store header
    WriteDLHeader('LE', Format('Logged continuously %s.', [setting]));
    Flush(DLRecFile);
    CloseFile(DLRecFile);
    LCLogFileCount := 1;
    FilesLogged.Caption := IntToStr(LCLogFileCount);
    Labelfile.Caption := 'file';

    //Save initial log time for 24 rollover check later
    LCStartFileTime := ThisMoment;

    LogOneReading; //First reading

  except
    StatusMessage('ERROR! IORESULT: ' + IntToStr(IOResult));
  end;

  //Set length of timer and start it
  FineTimer.Interval := 1000 - MilliSecondOf(Now);
  FineTimer.Enabled := True;

end;

procedure TFormLogCont.StopButtonClick(Sender: TObject);
begin
  StopRecording := True;
end;

procedure TFormLogCont.HotkeyStringGridKeyUp(Sender: TObject;
  var Key: word; Shift: TShiftState);
var
  i: integer;

  function StrippedOfNonAscii(const s: string): string;
  var
    i, Count: integer;
  begin
    SetLength(Result, Length(s));
    Count := 0;
    for i := 1 to Length(s) do
    begin
      if ((s[i] >= #32) and (s[i] < #127)) or (s[i] in [#10, #13]) then
      begin
        Inc(Count);
        Result[Count] := s[i];
      end;
    end;
    SetLength(Result, Count);
  end;

begin
  if HotkeyStringGrid.Col = 0 then
  begin
    HotkeyStringGrid.Cells[HotkeyStringGrid.Col, HotkeyStringGrid.Row] := keyslist(Key);
    HotKeyCodes[HotkeyStringGrid.Row] := Key;
  end;

  for i := 1 to HotkeyStringGrid.RowCount - 1 do
  begin
    vConfigurations.WriteString('HotKeys', 'h' + IntToStr(i - 1),
      IntToStr(HotKeyCodes[i]));
    vConfigurations.WriteString('HotKeys', 'a' + IntToStr(i - 1),
      StrippedOfNonAscii(HotkeyStringGrid.Cells[1, i]));
  end;
end;

procedure TFormLogCont.HotkeyStringGridSelectEditor(Sender: TObject;
  aCol, aRow: integer; var Editor: TWinControl);
begin
  if Editor is TStringCellEditor then
    CustomizeEdit(aCol, aRow, Editor as TStringCellEditor);
end;

procedure TFormLogCont.CheckSynchronized();
begin
  if (SynchronizedCheckBox.Checked) then
  begin
    PendingHotKey.Enabled := True;
    PendingLabel.Enabled := True;
    vConfigurations.WriteBool('HotKeys', 'sync', True);
  end
  else
  begin
    PendingHotKey.Enabled := False;
    PendingLabel.Enabled := False;
    vConfigurations.WriteBool('HotKeys', 'sync', False);
  end;
end;

procedure TFormLogCont.SynchronizedCheckBoxChange(Sender: TObject);
begin
  CheckSynchronized();
end;

procedure TFormLogCont.CustomizeEdit(ACol, ARow: integer; Editor: TStringCellEditor);
begin
  if aCol = 1 then
    Editor.MaxLength := 0;

end;


initialization
  {$I logcont.lrs}

end.
